import xml.etree.ElementTree as ElementTree
import urllib.parse
from .BaseClient import BaseClient
from ..parsers.parse_xml_to_json import parse_xml_to_json


class PrestaShopClient(BaseClient):
    def __init__(self, base_url, token):
        self.token = token
        self.base_url = base_url

    def build_url(self, url, params=None):
        updated_params = {**(params or {}), 'ws_key': self.token}
        return f'{self.base_url}/api{url}?{urllib.parse.urlencode(updated_params)}'

    def request(self, url, params=None, timeout=20):
        constructed_params = {'display': 'full', **(params or {})}
        full_url = self.build_url(url, constructed_params)
        xml_content = self.get(full_url, timeout=timeout)
        xml = ElementTree.fromstring(xml_content)
        result = [parse_xml_to_json(child) for child in list(xml)[0]]
        return result

    def get_products(self, params=None):
        constructed_params = {
            'display': 'full',
            **(params or {}),
        }
        full_url = self.build_url('/products', constructed_params)
        # we need to parse these since the response is in XML
        # check parsers folder for relevant functions/classes
        products = self.get(full_url)

        xml_products = ElementTree.fromstring(products)
        products = list(xml_products)[0]
        result = [parse_xml_to_json(child) for child in products]

        return result

    def get_categories(self, params=None):
        return self.request('/categories', params)

    def get_features(self, params=None):
        return self.request('/product_features', params)
    def get_feature_values(self, params=None):
        return self.request('/product_feature_values', params)

    def get_combinations(self, params=None):
        return self.request('/combinations', params)

    def get_product_options(self, params=None):
        return self.request('/product_options', params)

    def get_product_option_values(self, params=None):
        return self.request('/product_option_values', params)

    def get_images(self, params=None):
        return self.request('/images', params)

    def get_tax_rule_groups(self, params=None):
        return self.request('/tax_rule_groups', params)

    def get_tax_rules(self, params=None):
        return self.request('/tax_rules', params)

    def get_taxes(self, params=None):
        return self.request('/taxes', params)

    def get_specific_prices(self, params=None):
        return self.request('/specific_prices', params)
